using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using MeshVR;
using MVR.FileManagementSecure;
using MVR.Hub;
using SimpleJSON;
using UnityEngine;
using UnityEngine.UI;

namespace CheesyFX
{
    public class CheesyPluginSuite : MVRScript
    {
	    public static CheesyPluginSuite singleton;
	    private static List<Plugin> plugins = new List<Plugin>();
        private static PluginPreset femalePreset;
        private static PluginPreset malePreset;

        private static UIDynamicTabBar tabbar;
        private static List<object> UIElements = new List<object>();

        public static bool isInVar;

        private void OnEnable()
        {
            SuperController.singleton.onAtomAddedHandlers += AddPersonCallback;
            SuperController.singleton.onSceneLoadedHandlers += RegisterPersons;
        }

        public override void Init()
        {
	        singleton = this;
            isInVar = this.IsInVarPackage();
            plugins.Add(new Plugin("Custom/Scripts/CheesyFX/ShakeIt/ShakeIt.cslist"));
            plugins.Add(new Plugin("Custom/Scripts/CheesyFX/BodyLanguage/BodyLanguage.cslist"));
            plugins.Add(new Plugin("Custom/Scripts/CheesyFX/HeelAdjust2/HeelAdjust2.cslist"));

            femalePreset = new PluginPreset("female");
            malePreset = new PluginPreset("male");
			
            RegisterPersons();
            CreateUI();
        }

        private void Start()
        {
	        DownloadLatestVars();
        }

        public void CreateUI()
        {
	        var tabs = plugins.Select(x => x.name).ToList();
	        tabs.AddRange(new[] { "femalePreset", "malePreset" });
	        tabbar = CreateTabBar(tabs, SelectTab);
	        tabbar.SelectTab(0);
        }

        private void SelectTab(int id)
        {
	        RemoveUIElements(UIElements);
	        if(id < plugins.Count) UIElements = plugins[id].CreateUI();
	        else if (id == plugins.Count) UIElements = femalePreset.CreateUI();
	        else UIElements = malePreset.CreateUI();
        }
        
        public void OnDisable()
        {
            SuperController.singleton.onAtomAddedHandlers -= AddPersonCallback;
            SuperController.singleton.onSceneLoadedHandlers -= RegisterPersons;
        }

        public void OnDestroy()
        {
            SuperController.singleton.onAtomAddedHandlers -= AddPersonCallback;
            SuperController.singleton.onSceneLoadedHandlers -= RegisterPersons;
        }

        public static void DownloadLatestVars()
        {
	        foreach (var plugin in plugins)
	        {
		        if(plugin.autoDownload.val) plugin.DownloadVar();
	        }
	        // if(plugins[1].autoDownload.val) HubDownloader.singleton.FindPackage($"CheesyFX.BodyLanguage_Resources.latest", plugins[1].autoDownload.val);
        }

        private static void RegisterPersons()
        {
	        RegisterPersons(false);
        }

        public static void RegisterPersons(bool forceVar = false)
        {
            foreach (var atom in SuperController.singleton.GetAtoms())
            {
                 AddPerson(atom, forceVar);
            }
        }

        private static void AddPersonCallback(Atom atom)
        {
	        if(SuperController.singleton.isLoading) return;
	        AddPerson(atom);
        }

        private static void AddPerson(Atom atom, bool forceVar = false)
        {
            if (atom.type != "Person") return;
            singleton.StartCoroutine(AddPlugins(atom, forceVar));
        }

        public static IEnumerator AddPlugins(Atom person, bool forceVar = false)
        {
	     //    person.RegisterAdditionalStorable()
		    // person.UnregisterAdditionalStorable();
		    while (SuperController.singleton.isLoading)
		    {
			    yield return null;
			    // yield return new WaitForSeconds(.5f);
		    }
            yield return new WaitForEndOfFrame();
            yield return new WaitForEndOfFrame();
            yield return new WaitForEndOfFrame();
            yield return new WaitForEndOfFrame();

            var dcs = person.GetComponentInChildren<DAZCharacterSelector>();
            while ((object)dcs == null)
            {
                yield return new WaitForSeconds(.5f);
                dcs = person.GetComponentInChildren<DAZCharacterSelector>();
            }
            
            while (!dcs.selectedCharacter.ready)
            {
                yield return new WaitForSeconds(.5f);
            }
            var pm = person.GetComponentInChildren<MVRPluginManager>();
            var MVRScripts = pm.GetComponentsInChildren<MVRScript>();

            try
            {
	            foreach (var plugin in plugins)
	            {
		            plugin.GetLatestUrl();
		            plugin.presets.Clear();
		            MVRScript script = MVRScripts.FirstOrDefault(x => x.name.EndsWith(plugin.modules[0]));
		            plugin.loaded = script != null;
		            if (plugin.loaded && plugin.autoUpdate.val)
		            {
			            var id = script.name.Substring(0, script.name.IndexOf('_'));
			            string path = (string) pm.GetJSON()["plugins"][id];
			            bool needsUpdate = forceVar;
			            // if (!isInVar) needsUpdate = true;
			            if (isInVar)
			            {
				            path = path.Replace($"{plugin.packageName}.", "");
				            int version = int.Parse(path.Substring(0, path.IndexOf(":/")));
				            needsUpdate = plugin.latest > version;
			            }
		            
			            if (needsUpdate)
			            {
				            if(plugin.parseSettings.val)
				            {
					            foreach (var module in plugin.modules)
					            {
						            plugin.presets.Add(MVRScripts.FirstOrDefault(x => x.name.EndsWith($"CheesyFX.{module}"))
							            ?.GetJSON().AsObject);
						            // module.Print();
					            }
				            }
				            pm.RemovePluginWithUID(id);
				            plugin.loaded = false;
				            MVRScripts = pm.GetComponentsInChildren<MVRScript>();
			            
			            }
		            }
	            }
            }
            catch (Exception e)
            {
	            SuperController.LogError(e.ToString());
	            throw;
            }

            var gender = dcs.gender;
            foreach (var plugin in plugins)
            {
	            
	            var load = !plugin.loaded && (plugin.loadOn.val == "All" ||
                               (gender == DAZCharacterSelector.Gender.Female && plugin.loadOn.val == "Females") ||
                               (gender == DAZCharacterSelector.Gender.Male && plugin.loadOn.val == "Males"));
	            if (load)
	            {
		            
		            var script = pm.CreatePlugin();
		            script.pluginURLJSON.val = isInVar || forceVar || plugin.useVar.val? plugin.packageUrl : plugin.baseUrl;
		            
		            for (var i = 0; i < plugin.presets.Count; i++)
		            {
			            // if (plugin.presets[i] != null) script.scriptControllers[i].gameObject.GetComponent<MVRScript>().SendMessage("Load", plugin.presets[i], SendMessageOptions.DontRequireReceiver);
			            if (plugin.presets[i] != null)
			            {
				            script.scriptControllers[i].gameObject.GetComponent<MVRScript>().LateRestoreFromJSON(plugin.presets[i]);
			            }
		            }
		            if (!plugin.loadEnabled.val) script.scriptControllers.ForEach(x => x.script.enabledJSON.val = false);
	            }
            }
            var pluginPresetManager = person.GetStorableByID("PluginPresets").GetComponentInChildren<PresetManager>();
            var pluginPresetManagerControl = person.presetManagerControls.Find(pmc => pmc.name == "PluginPresets");
            if (gender == DAZCharacterSelector.Gender.Female && femalePreset.url.val != "")
            {
	            femalePreset.LoadJSON(femalePreset.url.val);
	            if (femalePreset.jc != null) pluginPresetManager.LoadPresetFromJSON(femalePreset.jc, true);
            }
            else if (gender == DAZCharacterSelector.Gender.Male && malePreset.url.val != "")
            {
	            malePreset.LoadJSON(femalePreset.url.val);
	            if (malePreset.jc != null) pluginPresetManager.LoadPresetFromJSON(malePreset.jc, true);
            }
        }

        public static string GetLatestPackageUid(string packageUid){
	        int version = FileManagerSecure.GetPackageVersion(packageUid+".latest");
	        if (version == -1) return null;
	        return $"{packageUid}.{version}";
        }

        private JSONClass Store()
        {
	        var jc = new JSONClass();
	        foreach (var plugin in plugins)
	        {
		        jc[plugin.name] = plugin.Store();
	        }
	        femalePreset.url.Store(jc);
	        femalePreset.useLatest.Store(jc);
	        malePreset.url.Store(jc);
	        malePreset.useLatest.Store(jc);
	        femalePreset.logUpdates.Store(jc);
	        malePreset.logUpdates.Store(jc);
	        return jc;
        }
        
        private void Load(JSONClass jc)
        {
	        foreach (var plugin in plugins)
	        {
		        if(jc.HasKey(plugin.name)) plugin.Load(jc[plugin.name].AsObject);
	        }
	        femalePreset.url.Load(jc);
	        femalePreset.useLatest.Load(jc);
	        malePreset.url.Load(jc);
	        malePreset.useLatest.Load(jc);
	        femalePreset.logUpdates.Load(jc);
	        malePreset.logUpdates.Load(jc);
        }

        public override JSONClass GetJSON(bool includePhysical = true, bool includeAppearance = true, bool forceStore = false)
        {
	        JSONClass jc = base.GetJSON(includePhysical, includeAppearance, true);
	        jc["plugins"] = Store();
	        return jc;
        }

        public override void LateRestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true,
	        bool setMissingToDefault = true)
        {
	        base.LateRestoreFromJSON(jc, restorePhysical, restoreAppearance, setMissingToDefault);
	        if(jc.HasKey("plugins")) Load(jc["plugins"].AsObject);
        }

        public UIDynamicTabBar CreateTabBar(List<string> menuItems, Action<int> selectTab, int columns = -1,
            bool invokeDefault = false, float width = 1f, float itemHeight = 50f)
        {
            if (columns == -1) columns = menuItems.Count;
            float rowCount = Mathf.Ceil((float)menuItems.Count / columns);
            float totalHeight = 10f + itemHeight * (rowCount);
            UIDynamicTabBar tabBar;
            GameObject tabbarPrefab = new GameObject("TabBar");
            RectTransform rt = tabbarPrefab.AddComponent<RectTransform>();
            rt.anchorMax = new Vector2(0, 1);
            rt.anchorMin = new Vector2(0, 1);
            rt.offsetMax = new Vector2(1064, -500);
            rt.offsetMin = new Vector2(10, -600);
            LayoutElement le = tabbarPrefab.AddComponent<LayoutElement>();
            le.flexibleWidth = 1;
            le.minHeight = totalHeight;
            le.minWidth = 1064;
            le.preferredHeight = totalHeight;
            le.preferredWidth = 1064;

            RectTransform backgroundTransform =
                manager.configurableScrollablePopupPrefab.transform.Find("Background") as RectTransform;
            backgroundTransform = Instantiate(backgroundTransform, tabbarPrefab.transform);
            backgroundTransform.name = "Background";
            backgroundTransform.anchorMax = new Vector2(1, 1);
            backgroundTransform.anchorMin = new Vector2(0, 0);
            //backgroundTransform.offsetMax = new Vector2(0, -15);
            //backgroundTransform.offsetMin = new Vector2(0, 15);

            UIDynamicTabBar uid = tabbarPrefab.AddComponent<UIDynamicTabBar>();
            float padding = 5.0f;
            float itemWidth, rowOffset;
            itemWidth = 1044f * width / columns - padding;

            float x = 15;
            for (int i = 0; i < menuItems.Count; ++i)
            {
                int rowNumber = i / columns;
                rowOffset = itemHeight * rowNumber;
                if (i % columns == 0) x = 15f;
                RectTransform buttonTransform =
                    manager.configurableScrollablePopupPrefab.transform.Find("Button") as RectTransform;
                buttonTransform = UnityEngine.Object.Instantiate(buttonTransform, tabbarPrefab.transform);
                buttonTransform.name = "Button";
                buttonTransform.anchorMax = new Vector2(0, 1);
                buttonTransform.anchorMin = new Vector2(0, 1);
                buttonTransform.offsetMax = new Vector2(x + itemWidth, -10 - rowOffset);
                buttonTransform.offsetMin = new Vector2(x, -itemHeight - rowOffset);
                Button buttonButton = buttonTransform.GetComponent<Button>();
                uid.buttons.Add(buttonButton);
                Text buttonText = buttonTransform.Find("Text").GetComponent<Text>();
                buttonText.text = menuItems[i];
                x += itemWidth + padding;
            }
            
            Transform t = CreateUIElement(tabbarPrefab.transform, false);
            // Transform t = script.CreateUIElement(tabbarPrefab.transform, false);
            tabBar = t.gameObject.GetComponent<UIDynamicTabBar>();
            for (int i = 0; i < tabBar.buttons.Count; ++i)
            {
                int menuID = i;
                tabBar.buttons[i].onClick.AddListener(() =>
                {
                    tabBar.lastId = tabBar.id;
                    tabBar.id = menuID;
                    for (int j = 0; j < tabBar.buttons.Count; ++j) tabBar.buttons[j].interactable = (j != menuID);
                    selectTab(menuID);
                });
                ColorBlock cb = ColorBlock.defaultColorBlock;
                cb.disabledColor = new Color(0.55f, 0.90f, 1f);
                cb.highlightedColor *= .8f;
                tabBar.buttons[i].colors = cb;
            }

            Destroy(tabbarPrefab);
            tabBar.spacer = CreateSpacer(true);
            tabBar.spacer.height = totalHeight;
            tabBar.SelectTab = val => tabBar.buttons[val].onClick.Invoke();
            tabBar.SelectLast = () => tabBar.SelectTab(tabBar.id);
            if (invokeDefault) tabBar.SelectTab(tabBar.id);
            return tabBar;
        }
        
        public void RemoveUIElements(List<object> UIElements)
		{
			for (int i=0; i<UIElements.Count; ++i)
			{
				if(UIElements[i] == null) continue;
				if (UIElements[i] is JSONStorableParam)
				{
					JSONStorableParam jsp = UIElements[i] as JSONStorableParam;
					if (jsp is JSONStorableFloat)
						RemoveSlider(jsp as JSONStorableFloat);
					else if (jsp is JSONStorableBool)
                        RemoveToggle(jsp as JSONStorableBool);
					else if (jsp is JSONStorableColor)
                        RemoveColorPicker(jsp as JSONStorableColor);
					else if (jsp is JSONStorableString)
                        RemoveTextField(jsp as JSONStorableString);
					else if (jsp is JSONStorableStringChooser)
					{
						// Workaround for VaM not cleaning its panels properly.
						JSONStorableStringChooser jssc = jsp as JSONStorableStringChooser;
						RectTransform popupPanel = jssc.popup?.popupPanel;
                        RemovePopup(jssc);
						if (popupPanel != null)
							Destroy(popupPanel.gameObject);
					}
				}
				else if (UIElements[i] is UIDynamic)
				{
					UIDynamic uid = UIElements[i] as UIDynamic;
					if (uid is UIDynamicButton)
                        RemoveButton(uid as UIDynamicButton);
                    else if (uid is UIDynamicSlider)
                        RemoveSlider(uid as UIDynamicSlider);
					else if (uid is UIDynamicToggle)
                        RemoveToggle(uid as UIDynamicToggle);
					else if (uid is UIDynamicColorPicker)
                        RemoveColorPicker(uid as UIDynamicColorPicker);
					else if (uid is UIDynamicTextField)
                        RemoveTextField(uid as UIDynamicTextField);
					else if (uid is UIDynamicPopup)
					{
						// Workaround for VaM not cleaning its panels properly.
						UIDynamicPopup uidp = uid as UIDynamicPopup;
						RectTransform popupPanel = uidp.popup?.popupPanel;
                        RemovePopup(uidp);
						if (popupPanel != null)
							Destroy(popupPanel.gameObject);
					}
                    RemoveSpacer(uid);
				}
			}
			UIElements.Clear();
		}
    }
    
    public class UIDynamicTabBar : UIDynamic
    {
	    public int id;
	    public int lastId;
	    public List<Button> buttons = new List<Button>();
	    public Action<int> SelectTab;
	    public Action SelectLast;
	    public UIDynamic spacer;

	    public void ToggleButtons(bool val, int id = -1)
	    {
		    if (id == -1)
		    {
			    for (int i=0; i<buttons.Count; i++) buttons[i].gameObject.SetActive(val);
			    return;
		    }
		    buttons[id].gameObject.SetActive(val);
	    }
    }
    
    public static class Extensions
    {
        public static bool IsInVarPackage(this MVRScript self)
        {
            string id = self.name.Substring(0, self.name.IndexOf('_'));
            string filename = self.manager.GetJSON()["plugins"][id].Value;
            return filename.IndexOf(":/") >= 0;
        }

        public static void Print(this object obj)
        {
            SuperController.LogMessage(obj.ToString());
        }
        
        public static void NullCheck(this object obj)
        {
	        SuperController.LogMessage((obj == null).ToString());
        }
        
        public static bool Store(this JSONStorableParam jsonParam, JSONClass jc, bool forceStore = true)
		{
			bool needsStore = false;
			if (jsonParam is JSONStorableBool)
			{
				JSONStorableBool jParam = (JSONStorableBool)jsonParam;
				if (forceStore || jParam.val != jParam.defaultVal)
				{
					jc[jParam.name].AsBool = jParam.val;
					needsStore = true;
				}
			}
			else if (jsonParam is JSONStorableFloat)
			{
				JSONStorableFloat jParam = (JSONStorableFloat)jsonParam;
				if (forceStore || jParam.val != jParam.defaultVal)
				{
					jc[jParam.name].AsFloat = jParam.val;
					needsStore = true;
				}
			}
			else if (jsonParam is JSONStorableString)
			{
				JSONStorableString jParam = (JSONStorableString)jsonParam;
				if (forceStore || jParam.val != jParam.defaultVal)
				{
					jc[jParam.name] = jParam.val;
					needsStore = true;
				}
			}
			else if (jsonParam is JSONStorableStringChooser)
			{
				JSONStorableStringChooser jParam = (JSONStorableStringChooser)jsonParam;
				if (forceStore || jParam.val != jParam.defaultVal)
				{
					jc[jParam.name] = jParam.val;
					needsStore = true;
				}
			}
			else if (jsonParam is JSONStorableColor)
			{
				JSONStorableColor jParam = (JSONStorableColor)jsonParam;
				JSONArray ja = new JSONArray();
				ja[0].AsFloat = jParam.val.H;
				ja[1].AsFloat = jParam.val.S;
				ja[2].AsFloat = jParam.val.V;
				jc[jParam.name] = ja;
				needsStore = true;
			}

			return needsStore;
		}

		public static void Load(this JSONStorableParam jsonParam, JSONClass jc)
		{
			if (jsonParam is JSONStorableBool)
			{
				JSONStorableBool jParam = (JSONStorableBool)jsonParam;
				if (jc.HasKey(jParam.name)) jParam.val = jc[jParam.name].AsBool;
			}
			else if (jsonParam is JSONStorableFloat)
			{
				JSONStorableFloat jParam = (JSONStorableFloat)jsonParam;
				if (jc.HasKey(jParam.name)) jParam.val = jc[jParam.name].AsFloat;
			}
			else if (jsonParam is JSONStorableString)
			{
				JSONStorableString jParam = (JSONStorableString)jsonParam;
				if (jc.HasKey(jParam.name)) jParam.val = jc[jParam.name];
			}
			else if (jsonParam is JSONStorableStringChooser)
			{
				JSONStorableStringChooser jParam = (JSONStorableStringChooser)jsonParam;
				if (jc.HasKey(jParam.name)) jParam.val = jc[jParam.name];
			}
			else if (jsonParam is JSONStorableColor)
			{
				JSONStorableColor jParam = (JSONStorableColor)jsonParam;
				if (jc.HasKey(jParam.name))
				{
					HSVColor loadedColor = new HSVColor
					{
						H = jc[jParam.name][0].AsFloat,
						S = jc[jParam.name][1].AsFloat,
						V = jc[jParam.name][2].AsFloat
					};
					jParam.val = loadedColor;
				}
			}
		}
		
		public static UIDynamic CreateUI(this JSONStorableParam jsonParam, bool rightSide = false, List<object> UIElements = null, int chooserType = 0)
		{
			UIDynamic uiDynamic = null;
			var script = CheesyPluginSuite.singleton;
			if (jsonParam is JSONStorableBool)
				uiDynamic = script.CreateToggle((JSONStorableBool)jsonParam, rightSide);
			else if (jsonParam is JSONStorableFloat)
				uiDynamic = script.CreateSlider((JSONStorableFloat)jsonParam, rightSide);
			else if (jsonParam is JSONStorableStringChooser)
			{
				if (chooserType == 0)
					uiDynamic = script.CreatePopup((JSONStorableStringChooser)jsonParam, rightSide);
				else if (chooserType == 1)
					uiDynamic = script.CreateScrollablePopup((JSONStorableStringChooser)jsonParam, rightSide);
				else if (chooserType == 2)
					uiDynamic = script.CreateFilterablePopup((JSONStorableStringChooser)jsonParam, rightSide);
			}
			else return null;

			if (UIElements != null) UIElements.Add(uiDynamic);
			return uiDynamic;
		}
    }
}