using System;
using System.Collections.Generic;
using System.Linq;
using MVR.FileManagementSecure;
using MVR.Hub;
using SimpleJSON;

namespace CheesyFX
{
    public class Plugin
    {
        public string name;
        public bool loaded;
        public string packageName =>  $"CheesyFX.{name}";
        public int latest;
        public List<string> modules;
        public List<JSONClass> presets = new List<JSONClass>();

        public JSONStorableStringChooser loadOn = new JSONStorableStringChooser("loadOn",
            new List<string> { "All", "Females", "Males", "Nobody"}, "All", "Load On");
        public JSONStorableBool loadEnabled = new JSONStorableBool("Load Enabled", true);
        public JSONStorableBool autoDownload = new JSONStorableBool("Auto Download", true);
        public JSONStorableBool autoUpdate = new JSONStorableBool("Auto Update", true);
        public JSONStorableBool parseSettings = new JSONStorableBool("Keep Settings On Update", true);
        public JSONStorableBool useVar = new JSONStorableBool("Use Var", false);

        public string packageUrl;
        public string baseUrl;

        public Plugin(string baseUrl)
        {
            this.baseUrl = baseUrl;
            name = baseUrl.Replace("Custom/Scripts/CheesyFX/", "");
            name = name.Substring(0, name.IndexOf("/"));
            modules = new List<string> { name };
            switch (name)
            {
                case "ShakeIt":
                {
                    modules = new List<string> {"ShakeIt", "FlexIt", "WorkOut"};
                    break;
                }
                case "BodyLanguage":
                {
                    modules = new List<string> {"FillMeUp", "TouchMe", "ReadMyLips", "WatchMe", "PoseMe"};
                    loadOn.choices = new List<string> { "Females", "Nobody" };
                    loadOn.val = "Females";
                    break;
                }
                case "HeelAdjust2":
                {
                    loadOn.val = "Females";
                    break;
                }
            }
            // GetLatestUrl();
        }

        public List<object> CreateUI()
        {
            List<object> UIElements = new List<object>();
            autoDownload.CreateUI(UIElements:UIElements);
            autoUpdate.CreateUI(UIElements:UIElements);
            var button = CheesyPluginSuite.singleton.CreateButton("Download Latest");
            button.button.onClick.AddListener(() => DownloadVar(true));
            UIElements.Add(button);
            button = CheesyPluginSuite.singleton.CreateButton("Update Persons");
            button.button.onClick.AddListener(() => CheesyPluginSuite.RegisterPersons());
            UIElements.Add(button);
            loadEnabled.CreateUI(true, UIElements:UIElements);
            parseSettings.CreateUI(true, UIElements:UIElements);
            var chooser = (UIDynamicPopup)loadOn.CreateUI(UIElements:UIElements);
            chooser.popupPanelHeight = 275f;
            if(!CheesyPluginSuite.isInVar)
            {
                button = CheesyPluginSuite.singleton.CreateButton("SwitchToVar", true);
                button.button.onClick.AddListener(() => CheesyPluginSuite.RegisterPersons(true));
                UIElements.Add(button);
                useVar.CreateUI(true, UIElements:UIElements);
            }
            return UIElements;
        }

        public JSONClass Store()
        {
            var jc = new JSONClass();
            loadOn.Store(jc);
            loadEnabled.Store(jc);
            autoDownload.Store(jc);
            autoUpdate.Store(jc);
            parseSettings.Store(jc);
            if(!CheesyPluginSuite.isInVar) useVar.Store(jc);
            return jc;
        }
        
        public void Load(JSONClass jc)
        {
            loadOn.Load(jc);
            loadEnabled.Load(jc);
            autoDownload.Load(jc);
            autoUpdate.Load(jc);
            parseSettings.Load(jc);
            if(!CheesyPluginSuite.isInVar) useVar.Load(jc);
        }

        public void DownloadVar(bool forceDownload = false)
        {
            HubDownloader.singleton.FindPackage($"{packageName}.latest", forceDownload || autoDownload.val);
            if (name.Contains("BodyLanguage"))
                HubDownloader.singleton.FindPackage($"CheesyFX.BodyLanguage_Resources.latest", forceDownload || autoDownload.val);
        }

        public void GetLatestUrl()
        {
            string packageUid = CheesyPluginSuite.GetLatestPackageUid(packageName);
            if (packageUid != null)
            {
                latest = int.Parse(packageUid.Substring(packageUid.LastIndexOf(".") + 1));
                packageUrl = packageUid + ":/" + baseUrl;
                if (name == "ShakeIt" && !FileManagerSecure.FileExists(packageUrl))
                    packageUrl = packageUrl.Replace("CheesyFX/ShakeIt/ShakeIt.cslist", "CheesyFX/ShakeIt.cslist");
            }
            else
            {
                loadOn.val = $"{packageName} missing";
                loadOn.choices = new List<string> { $"{packageName} missing" };
            }
        }
    }
}