using System;
using System.Collections.Generic;
using System.Linq;
using SimpleJSON;

namespace CheesyFX
{
    public class PluginPreset
    {
        public JSONStorableUrl url;
        public JSONClass jc;
        public JSONStorableBool useLatest = new JSONStorableBool("Use Latest vars", true);
        public JSONStorableBool logUpdates = new JSONStorableBool("Log Updates", true);

        public PluginPreset(string gender)
        {
            url = new JSONStorableUrl($"{gender}Url", "", "vap", "Custom/Atom/Person/Plugins");
            useLatest.name += $" ({gender})";
            logUpdates.name += $" ({gender})";
            url.hideExtension = true;
            url.allowFullComputerBrowse = false;
            url.allowBrowseAboveSuggestedPath = true;
            url.setCallbackFunction += LoadJSON;
            useLatest.setCallbackFunction += val => LoadJSON(url.val);
        }

        public void LoadJSON(string url)
        {
            if (this.url.val == "")
            {
                jc = null;
                return;
            }
            jc = SuperController.singleton.LoadJSON(url).AsObject;
            if(useLatest.val) UpdateToLatest();
        }

        private void OverrideJSON()
        {
            SuperController.singleton.SaveJSON(jc, url.val);
        }

        private void UpdateToLatest()
        {
            try
            {
                if (jc == null) return;
                var plugins = jc["storables"].AsArray[0]["plugins"].AsObject;
                var tempPlugins = new JSONClass();
                foreach (var id in plugins.Keys)
                {
                    string uid = plugins[id];
                    string storedPackage = uid.Substring(0, uid.IndexOf(":/"));
                    var baseName = storedPackage.Substring(0, storedPackage.LastIndexOf("."));
                    var newPackage = CheesyPluginSuite.GetLatestPackageUid(baseName);
                    tempPlugins[id] = uid.Replace(storedPackage, newPackage);
                    if (logUpdates.val && storedPackage != newPackage)
                    {
                        $"CheesyPluginSuit: {newPackage} will be added instead of {storedPackage}".Print();
                    } 
                }
                jc["storables"].AsArray[0]["plugins"] = tempPlugins;
            }
            catch (Exception e)
            {
                SuperController.LogError(e.ToString());
                throw;
            }

            
        }

        public List<object> CreateUI()
        {
            List<object> UIElements = new List<object>();
            var button = CheesyPluginSuite.singleton.CreateButton("Browse");
            url.RegisterFileBrowseButton(button.button);
            UIElements.Add(button);
            button = CheesyPluginSuite.singleton.CreateButton("Reset");
            button.button.onClick.AddListener(() => url.val = "");
            UIElements.Add(button);
            var textField = CheesyPluginSuite.singleton.CreateTextField(url, true);
            UIElements.Add(textField);
            useLatest.CreateUI(UIElements:UIElements);
            logUpdates.CreateUI(UIElements:UIElements);
            button = CheesyPluginSuite.singleton.CreateButton("Override preset (updated)");
            button.button.onClick.AddListener(OverrideJSON);
            UIElements.Add(button);
            return UIElements;
        }
    }
}